import { world, system, ItemStack, Items, Direction, Vector, MinecraftEntityTypes, MinecraftBlockTypes, Location, BlockLocation, BlockAreaSize, MinecraftEffectTypes, EntityDamageCause } from "@minecraft/server"
let createFondaQueue = []
let replaceSlotQueue = []
let loadWorldTagPlayer = undefined
let worldInitialized = false
let worldTick = 0
const timer = async (/** @type {number} */ ms) => {
    const resolved = Promise.resolve();
    const date = Date.now();
    do {
        await resolved;
    } while (Date.now() - date < ms);
}
const dimensionList = [
    world.getDimension('overworld'),
    world.getDimension('nether'),
    world.getDimension('the_end')
]
const fakeEmptySlot = new ItemStack(Items.get('chileancraft:empty_slot'), 1, 0)
const liquidTypeItem = new ItemStack(Items.get('chileancraft:liquid_icon'), 1, 0)
const replaceableBlocks = [
    'minecraft:air',
    'minecraft:tallgrass'
]
const chileanComposter = {
    'chileancraft:black_parra_seeds': 0.3,
    'chileancraft:cebolla_seeds': 0.3,
    'chileancraft:choclo_seeds': 0.3,
    'chileancraft:green_aji_seeds': 0.3,
    'chileancraft:green_parra_seeds': 0.3,
    'chileancraft:red_aji_seeds': 0.3,
    'chileancraft:red_parra_seeds': 0.3,
    'chileancraft:tomate_seeds': 0.3,
    'chileancraft:pinones': 0.3,
    'chileancraft:araucaria_leaves': 0.3,
    'chileancraft:espino_leaves': 0.3,
    'chileancraft:cooked_pinones': 0.5,
    'chileancraft:tuna.item': 0.5,
    'chileancraft:san_pedro': 0.5,
    'chileancraft:red_aji': 0.5,
    'chileancraft:green_aji': 0.5,
    'chileancraft:green_grapes': 0.65,
    'chileancraft:red_grapes': 0.65,
    'chileancraft:black_grapes': 0.65,
    'chileancraft:tuna_fruit': 0.65,
    'chileancraft:tomate': 0.65,
    'chileancraft:choclo': 0.65,
    'chileancraft:cebolla': 0.65,
    'chileancraft:ananuca.item': 0.65,
    'chileancraft:bailahuen.item': 0.65,
    'chileancraft:cachiyuyo.item': 0.65,
    'chileancraft:cardo.item': 0.65,
    'chileancraft:diente_leon.item': 0.65,
    'chileancraft:doca.item': 0.65,
    'chileancraft:manzanilla.item': 0.65,
    'chileancraft:pata_guanaco.item': 0.65,
    'chileancraft:ruda.item': 0.65,
    'chileancraft:san_juan_plant.item': 0.65,
    'chileancraft:small_cactus.item': 0.65,
    'chileancraft:cooked_choclo': 0.85,
    'chileancraft:cooked_cebolla': 0.85
}
const chileanSeeds = [
    'chileancraft:black_parra_seeds',
    'chileancraft:cebolla_seeds',
    'chileancraft:choclo_seeds',
    'chileancraft:green_aji_seeds',
    'chileancraft:green_parra_seeds',
    'chileancraft:red_aji_seeds',
    'chileancraft:red_parra_seeds',
    'chileancraft:tomate_seeds',
]
const chileanFlowers = [
    'chileancraft:bailahuen',
    'chileancraft:cardo',
    'chileancraft:diente_leon',
    'chileancraft:manzanilla',
    'chileancraft:pata_guanaco',
    'chileancraft:san_juan_plant'
]
const digRaycastQuery = {
    includeLiquidBlocks: false,
    includePassableBlocks: true,
    maxDistance: 6
}
const vanillaBlocksPick = [
    'minecraft:ice',
    'minecraft:packed_ice',
    'minecraft:blue_ice',
    'minecraft:coal_ore',
    'minecraft:deepslate_coal_ore',
    'minecraft:coal_block',
    'minecraft:iron_ore',
    'minecraft:deepslate_iron_ore',
    'minecraft:iron_block',
    'minecraft:emerald_ore',
    'minecraft:deepslate_emerald_ore',
    'minecraft:emerald_block',
    'minecraft:redstone_ore',
    'minecraft:deepslate_redstone_ore',
    'minecraft:redstone_block',
    'minecraft:gold_ore',
    'minecraft:deepslate_gold_ore',
    'minecraft:gold_block',
    'minecraft:lapis_ore',
    'minecraft:deepslate_lapis_ore',
    'minecraft:lapis_block',
    'minecraft:copper_ore',
    'minecraft:deepslate_copper_ore',
    'minecraft:copper_block',
    'minecraft:diamond_ore',
    'minecraft:deepslate_diamond_ore',
    'minecraft:diamond_block',
    'minecraft:sandstone',
    'minecraft:sandstone_stairs',
    'minecraft:smooth_sandstone_stairs',
    'minecraft:red_sandstone',
    'minecraft:red_sandstone_stairs',
    'minecraft:smooth_red_sandstone_stairs',
    'minecraft:anvil',
    'minecraft:concrete',
    'minecraft:hardened_clay',
    'minecraft:stained_hardened_clay',
    'minecraft:observer',
    'minecraft:grindstone',
    'minecraft:calcite',
    'minecraft:tuff',
    'minecraft:netherrack',
    'minecraft:purpur_block',
    'minecraft:purpur_stairs',
    'minecraft:end_stone',
    'minecraft:prismarine',
    'minecraft:prismarine_stairs',
    'minecraft:dark_prismarine_stairs',
    'minecraft:bone_block',
    'minecraft:lodestone',
    'minecraft:dripstone',
    'minecraft:pointed_dripstone',
    'minecraft:stonecutter'
]
const vanillaTilePick = [
    'minecraft:smoker',
    'minecraft:furnace',
    'minecraft:blast_furnace',
    'minecraft:cauldron',
    'minecraft:hopper',
    'minecraft:dropper',
    'minecraft:dispenser',
]
const vanillaBlocksShovel = [
    'minecraft:dirt_with_roots',
    'minecraft:clay'
]
const pickDigEffectCustom = {
    wooden: (player, block) => {
        if (block.hasTag('stone') || block.hasTag('wooden_pick_diggable')) player.addEffect(MinecraftEffectTypes.haste, 3, 7, false)
    },
    stone: (player, block) => {
        if (block.hasTag('stone') || block.hasTag('stone_pick_diggable') || block.hasTag('wooden_pick_diggable')) player.addEffect(MinecraftEffectTypes.haste, 3, 9, false)
    },
    golden: (player, block) => {
        if (block.hasTag('stone') || block.hasTag('golden_pick_diggable')) player.addEffect(MinecraftEffectTypes.haste, 3, 16, false)
    },
    iron: (player, block) => {
        if (block.hasTag('stone') || block.hasTag('metal') || block.hasTag('iron_pick_diggable') || block.hasTag('stone_pick_diggable') || block.hasTag('wooden_pick_diggable') || block.hasTag('golden_pick_diggable')) player.addEffect(MinecraftEffectTypes.haste, 3, 11, false)
    },
    diamond: (player, block) => {
        if (block.hasTag('stone') || block.hasTag('metal') || block.hasTag('diamond_pick_diggable') || block.hasTag('iron_pick_diggable') || block.hasTag('stone_pick_diggable') || block.hasTag('wooden_pick_diggable') || block.hasTag('golden_pick_diggable')) player.addEffect(MinecraftEffectTypes.haste, 3, 13, false)
    },
    netherite: (player, block) => {
        if (block.hasTag('stone') || block.hasTag('metal') || block.hasTag('netherite_pick_diggable') || block.hasTag('diamond_pick_diggable') || block.hasTag('iron_pick_diggable') || block.hasTag('stone_pick_diggable') || block.hasTag('wooden_pick_diggable') || block.hasTag('golden_pick_diggable')) player.addEffect(MinecraftEffectTypes.haste, 3, 15, false)
    }
}
const axeDigEffectCustom = {
    wooden: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 2, false),
    stone: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 3, false),
    golden: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 7, false),
    iron: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 4, false),
    diamond: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 5, false),
    netherite: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 6, false)
}
const shovelDigEffectCustom = {
    wooden: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 3, false),
    stone: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 5, false),
    golden: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 9, false),
    iron: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 5, false),
    diamond: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 7, false),
    netherite: (player) => player.addEffect(MinecraftEffectTypes.haste, 3, 8, false)
}
const toolType = [
    '_pickaxe',
    '_shovel',
    '_hoe',
    '_sword',
    '_axe'
]
const shovelDigTags = [
    'grass',
    'dirt',
    'gravel',
    'snow',
    'sand'
]
const hoeDigID = [
    'minecraft:sculk',
    'minecraft:sculk_vein',
    'minecraft:sculk_shrieker',
    'minecraft:sculk_sensor',
    'minecraft:sculk_catalyst',
    'minecraft:nether_wart_block',
    'minecraft:warped_wart_block',
    'minecraft:hay_block',
    'minecraft:target',
    'minecraft:dried_kelp_block',
    'minecraft:shroomlight',
    'minecraft:sponge',
    'minecraft:moss_block'
]
const vanillaBlocksSword = [
    'minecraft:web',
    'minecraft:bamboo'
]
// tag search
const tagSearch = (entity, id) => {
    for (const tag of entity.getTags()) {
        if (tag.startsWith(id)) {
            return JSON.parse(tag.replace(id, ''))
        }
    }
}
// tool func
const toolDigFunction = {
    pickaxe: (player, item, block) => {
        const tier = item.typeId.split(':')[1].replace('_pickaxe', '')
        if (!block.typeId.includes('minecraft:')) pickDigEffectCustom[tier](player, block)
    },
    shovel: (player, item, block) => {
        for (const currentTag of shovelDigTags) {
            if (!block.typeId.includes('minecraft:') && block.hasTag(currentTag)) {
                const tier = item.typeId.split(':')[1].replace('_shovel', '')
                shovelDigEffectCustom[tier](player)
                break;
            }
        }
    },
    hoe: (player, item, block) => {
        const tier = item.typeId.split(':')[1].replace('_hoe', '')
    },
    sword: (player, item, block) => {
        const tier = item.typeId.split(':')[1].replace('_sword', '')
    },
    axe: (player, item, block) => {
        const tier = item.typeId.split(':')[1].replace('_axe', '')
        if (!block.typeId.includes('minecraft:') && (block.hasTag('wood') || block.hasTag('pumpkin'))) axeDigEffectCustom[tier](player)
    }
}
const toolDamageFunction = {
    pickaxe: async (player, inventory, slot, item, block, oldBlock) => {
        let efficient = false
        let durability = item.getComponent('durability')
        switch (item.typeId.split(':')[1].replace('_pickaxe', '')) {
            case 'copper':
                efficient = (block.hasTag('stone') || block.hasTag('metal') || block.hasTag('iron_pick_diggable') || block.hasTag('stone_pick_diggable') || block.hasTag('wooden_pick_diggable') || block.hasTag('golden_pick_diggable') || vanillaBlocksPick.indexOf(block.type.id) > -1 || block.type.id.includes('deepslate') || block.type.id.includes('blackstone') || block.type.id.includes('basalt') || block.type.id.includes('amethyst') || block.type.id.includes('brick') || block.type.id.includes('quartz') || block.type.id.includes('terracotta')) && vanillaTilePick.indexOf(block.typeId) === -1
                break;
            case 'charged_copper':
                efficient = (block.hasTag('stone') || block.hasTag('metal') || block.hasTag('iron_pick_diggable') || block.hasTag('stone_pick_diggable') || block.hasTag('wooden_pick_diggable') || block.hasTag('golden_pick_diggable') || vanillaBlocksPick.indexOf(block.type.id) > -1 || block.type.id.includes('deepslate') || block.type.id.includes('blackstone') || block.type.id.includes('basalt') || block.type.id.includes('amethyst') || block.type.id.includes('brick') || block.type.id.includes('quartz') || block.type.id.includes('terracotta')) && vanillaTilePick.indexOf(block.typeId) === -1
                break;
        }
        if (efficient) durability.damage++
        else durability.damage = durability.damage + 2
        if (durability.damage >= durability.maxDurability) {
            replaceSlotQueue.push({
                entity: player,
                inventory,
                slot,
                item: 'air'
            })
            player.runCommandAsync('playsound random.break @a ~~~')
        } else replaceSlotQueue.push({
            inventory,
            slot,
            item
        })
    },
    shovel: async (player, inventory, slot, item, block) => {
        let efficient = false
        let durability = item.getComponent('durability')
        switch (item.typeId.split(':')[1].replace('_shovel', '')) {
            case 'copper':
                efficient = (vanillaBlocksShovel.indexOf(block.type.id) > -1)
                if (!efficient) for (const currentTag of shovelDigTags) {
                    if (block.hasTag(currentTag)) {
                        efficient = true
                        break;
                    }
                }
                break;
        }
        if (efficient) durability.damage++
        else durability.damage = durability.damage + 2
        if (durability.damage >= durability.maxDurability) {
            replaceSlotQueue.push({
                entity: player,
                inventory,
                slot,
                item: 'air'
            })
            player.runCommandAsync('playsound random.break @a ~~~')
        } else replaceSlotQueue.push({
            inventory,
            slot,
            item
        })
    },
    hoe: async (player, inventory, slot, item, block) => {
        let efficient = (hoeDigID.indexOf(block.typeId) > -1)
        if (efficient) {
            let durability = item.getComponent('durability')
            durability.damage++
            if (durability.damage >= durability.maxDurability) {
                replaceSlotQueue.push({
                    entity: player,
                    inventory,
                    slot,
                    item: 'air'
                })
                player.runCommandAsync('playsound random.break @a ~~~')
            } else replaceSlotQueue.push({
                inventory,
                slot,
                item
            })
        }
    },
    sword: async (player, inventory, slot, item, block) => {
        let efficient = false
        let durability = item.getComponent('durability')
        switch (item.typeId.split(':')[1].replace('_sword', '')) {
            case 'copper':
                efficient = (vanillaBlocksSword.indexOf(block.type.id) > -1)
                break;
        }
        if (efficient) durability.damage++
        else durability.damage = durability.damage + 2
        if (durability.damage >= durability.maxDurability) {
            replaceSlotQueue.push({
                entity: player,
                inventory,
                slot,
                item: 'air'
            })
            player.runCommandAsync('playsound random.break @a ~~~')
        } else replaceSlotQueue.push({
            inventory,
            slot,
            item
        })
    },
    axe: async (player, inventory, slot, item, block) => {
        let efficient = false
        let durability = item.getComponent('durability')
        switch (item.typeId.split(':')[1].replace('_axe', '')) {
            case 'copper':
                efficient = (block.hasTag('wood') || block.hasTag('pumpkin') || block.hasTag('plant') || vanillaBlocksAxe.indexOf(block.type.id) > -1)
                break;
        }
        if (efficient) durability.damage++
        else durability.damage = durability.damage + 2
        if (durability.damage >= durability.maxDurability) {
            replaceSlotQueue.push({
                entity: player,
                inventory,
                slot,
                item: 'air'
            })
            player.runCommandAsync('playsound random.break @a ~~~')
        } else replaceSlotQueue.push({
            inventory,
            slot,
            item
        })

    }
}
const replaceSlot = async () => {
    for (const currentReplace of replaceSlotQueue) {
        if (currentReplace.item === 'air' && currentReplace.entity !== undefined) currentReplace.entity.runCommandAsync('replaceitem entity @s slot.weapon.mainhand 0 air')
        else currentReplace.inventory.setItem(currentReplace.slot, currentReplace.item)
        replaceSlotQueue.shift()
    }
}
const playerUpdate = async () => {
    for (const player of world.getPlayers()) {
        try {
            const item = player.getComponent('inventory').container.getItem(player.selectedSlot)
            if (item !== undefined) for (let currentTool of toolType) {
                if (item.typeId.includes(currentTool)) {
                    const block = player.getBlockFromViewDirection(digRaycastQuery)
                    if (block !== null) toolDigFunction[currentTool.replace('_', '')](player, item, block)
                    break;
                }
            }
        } catch (err) { }
    }

}
//door
const placeDoorTest = async (/** @type {import("@minecraft/server").BlockLocation} */ blockLocation, /** @type {any} */ blockFace, /** @type {import("@minecraft/server").Dimension} */ dimension, /** @type {{ rotation: { y: number; }; name: any; runCommandAsync: (arg0: string) => void; }} */ source, /** @type {any} */ item, /** @type {string} */ sound, /** @type {string} */ doorID, data) => {
    try {
        if (blockFace === Direction.up) {
            let doorBlock = dimension.getBlock(blockLocation.offset(0, 1, 0))
            if (doorBlock.typeId === 'minecraft:air') {
                const upBlock = dimension.getBlock(doorBlock.location.offset(0, 1, 0))
                if (upBlock.typeId !== 'minecraft:air') data.cancel = true
            }
        } else data.cancel = true
    } catch (err) { console.warn(err) }
}
const openDoor = (dimension, block, blockLocation) => {
    let permutation = block.permutation
    const properties = {
        top: permutation.getProperty('chileancraft:top').value,
        open: permutation.getProperty('chileancraft:open').value,
        direction: permutation.getProperty('chileancraft:direction').value
    }
    const direction = [0, 1, 2, 3]
    properties.direction = properties.direction + (properties.open ? -1 : 1)
    properties.direction = properties.direction === 4 ? 0 : (properties.direction === -1 ? 3 : properties.direction)
    properties.direction = direction[properties.direction]
    dimension.runCommandAsync(`playsound ${!properties.open ? 'open' : 'close'}.wooden_door @a ${block.location.x} ${block.location.y} ${block.location.z}`)
    setDoorPermutation((properties.top ? dimension.getBlock(blockLocation.offset(0, -1, 0)) : block), block.typeId, properties.direction, !properties.open)
}
const setDoorPermutation = (block, doorType, direction, side) => {
    let permutation = MinecraftBlockTypes.get(doorType).createDefaultBlockPermutation()
    permutation.getProperty('chileancraft:direction').value = direction
    permutation.getProperty('chileancraft:item').value = false
    permutation.getProperty('chileancraft:top').value = false
    permutation.getProperty('chileancraft:open').value = side
    block.setPermutation(permutation)
    permutation.getProperty('chileancraft:top').value = true
    block.dimension.getBlock(block.location.offset(0, 1, 0)).setPermutation(permutation)
}
//
const containerBlocks = [
    'minecraft:chest',
    'minecraft:trapped_chest',
    'chileancraft:red_clay_pig',
    'chileancraft:red_clay_pot',
    'chileancraft:red_clay_jar',
    'chileancraft:red_clay_small_jar'
]
const vanillaStrippableLogBlock = [
    'minecraft:log',
    'minecraft:log2',
    'minecraft:wood',
    'minecraft:mangrove_log',
    'minecraft:mangrove_wood',
    'minecraft:crimson_stem',
    'minecraft:crimson_hyphae',
    'minecraft:warped_stem',
    'minecraft:warped_hyphae'
]
// fonda data
const removeFonda = async (fondaData, location, dimension) => {
    await dimension.runCommandAsync(`event entity @e[type=chileancraft:red_clay_small_jar,c=1,tag=chileancraft_fonda_generated,r=10,x=${location.x},y=${location.y},z=${location.z}] chileancraft:despawn`)
    await dimension.runCommandAsync(`structure load mystructure:fonda_og_${fondaData.fondaID} ${location.x} ${location.y} ${location.z}`)
    await dimension.runCommandAsync(`structure delete mystructure:fonda_og_${fondaData.fondaID}`)
}
const createFonda = async (dimension, exactLocation, blockLocation, x, z) => {
    try {
        const fondaID = Math.round(Math.random() * 1000000)
        await dimension.runCommandAsync(`structure save mystructure:fonda_og_${fondaID} ${exactLocation.x} ${exactLocation.y} ${exactLocation.z} ${exactLocation.x + x + 1} ${exactLocation.y + 6} ${exactLocation.z + z + 1} false disk`)
        await dimension.runCommandAsync(`structure load mystructure:fonda ${exactLocation.x} ${exactLocation.y} ${exactLocation.z} ${Math.random > 0.5 ? (x === 10 ? 90 : 0) : (x === 10 ? 270 : 180)}_degrees`)
        const fondaVillager = dimension.spawnEntity('chileancraft:fonda_villager', blockLocation.offset(5, 2, 5))
        fondaVillager.triggerEvent('chileancraft:fonda_generated')
        fondaVillager.nameTag = '§2El fondero'
        fondaVillager.addTag(JSON.stringify({
            fondaID,
            location: {
                x: blockLocation.x,
                y: blockLocation.y,
                z: blockLocation.z
            },
            x,
            z,
            dimension: dimension.id,
            brokenBlockCount: 0
        }))
    } catch (err) { console.warn(err) }
}
const fondaAdvertMessages = [
    'Ya cortala',
    'Ultima y me voy',
    'Cagaste >:('
]
// events
const entityEvents = {
    'chileancraft:painting_destroy': (entity, dimension) => {
        try {
            const location = new Location(entity.location.x, entity.location.y, entity.location.z)
            const getPlayer = Array.from(dimension.getEntities({
                closest: 1,
                location: location,
                type: 'minecraft:player',
                tags: [
                    'breaker'
                ],
                gameMode: 1
            }))[0]
            if (getPlayer !== undefined) {
                getPlayer.removeTag('breaker')
            } else dimension.spawnItem(new ItemStack(Items.get('minecraft:painting'), 1, 0), location)
            dimension.runCommandAsync(`playsound block.itemframe.break @a ${location.x} ${location.y} ${location.z} 0.7`)
            dimension.runCommandAsync(`particle chileancraft:oak_planks ${location.x} ${location.y + 0.5} ${location.z}`)
            entity.triggerEvent('chileancraft:despawn')
        } catch (err) { console.warn(err) }

    },
    'minecraft:spawn_from_village': (entity, dimension) => {
        if (entity.typeId === 'minecraft:villager_v2' && Math.random() > 0.9) {
            const rng = Math.random()
            const location = new Location(entity.location.x, entity.location.y, entity.location.z)
            if (rng < 0.35) dimension.spawnEntity('chileancraft:fonda_villager', location)
            else if (rng < 0.7) dimension.spawnEntity('chileancraft:huaso_villager', location)
            else if (rng < 0.9) dimension.spawnEntity('chileancraft:ligua_villager', location)
            else if (rng < 0.97) dimension.spawnEntity('chileancraft:tio_aceite', location)
            else dimension.spawnEntity('chileancraft:haitiano_villager', location)
            entity.teleport(new Location(entity.location.x, -100, entity.location.z), dimension, 0, 0, false)
        }
    },
    'chileancraft:retire': (entity, dimension) => {
        for (let fondaData of entity.getTags()) {
            if (fondaData.startsWith('{"')) {
                fondaData = JSON.parse(fondaData)
                removeFonda(fondaData, new Location(fondaData.location.x, fondaData.location.y, fondaData.location.z), world.getDimension(fondaData.dimension))
                break
            }
        }
        entity.triggerEvent('chileancraft:despawn')
    },
    'chileancraft:important_block_broken': (entity, dimension) => {
        for (let fondaData of entity.getTags()) {
            if (fondaData.startsWith('{"')) {
                fondaData = JSON.parse(fondaData)
                for (const player of dimension.getEntities({
                    location: new Location(entity.location.x, entity.location.y, entity.location.z),
                    maxDistance: 10,
                    type: 'minecraft:player'
                })) {
                    player.tell('<Aldeano de fonda> Te pasai de lanza >:(')
                }
                entity.triggerEvent('chileancraft:despawn')
                removeFonda(fondaData, new Location(fondaData.location.x, fondaData.location.y, fondaData.location.z), world.getDimension(fondaData.dimension))
                break
            }
        }
        entity.triggerEvent('chileancraft:despawn')
    },
    'chileancraft:block_broken': (entity, dimension) => {
        for (let fondaData of entity.getTags()) {
            if (fondaData.startsWith('{"')) {
                entity.removeTag(fondaData)
                fondaData = JSON.parse(fondaData)
                for (const player of dimension.getEntities({
                    location: new Location(entity.location.x, entity.location.y, entity.location.z),
                    maxDistance: 10,
                    type: 'minecraft:player'
                })) {
                    player.tell(`<Aldeano de fonda> ${fondaAdvertMessages[fondaData.brokenBlockCount]}`)
                }
                if (fondaData.brokenBlockCount < 2) {
                    fondaData.brokenBlockCount++
                    entity.addTag(JSON.stringify(fondaData))
                } else {
                    entity.triggerEvent('chileancraft:despawn')
                    removeFonda(fondaData, new Location(fondaData.location.x, fondaData.location.y, fondaData.location.z), world.getDimension(fondaData.dimension))
                }
                break
            }
        }
    },
    'chileancraft:red_clay_pot_destroy': (entity, dimension) => {
        if (!entity.hasTag('chileancraft:destroy')) {
            entity.addTag('chileancraft:destroy')
            const /** @type {import("@minecraft/server").InventoryComponentContainer} */ container = entity.getComponent('inventory').container
            for (const i of [0, 11]) {
                const itemStack = container.getItem(i)
                if (itemStack !== undefined) {
                    const item = dimension.spawnItem(itemStack, new Location(entity.location.x, entity.location.y + 0.3, entity.location.z))
                    item.setVelocity(new Vector(0.2 - Math.random() * 0.4, 0.2, 0.2 - Math.random() * 0.4))
                }
            }
            world.scoreboard.removeObjective(`rcp_${entity.id.substring(entity.id.length - 10, entity.id.length)}`)
            entity.triggerEvent('chileancraft:despawn')

        }
    },
    'chileancraft:jukebox_destroy': async (entity, dimension) => {
        try {
            if (!entity.hasTag('chileancraft:destroy')) {
                entity.addTag('chileancraft:destroy')
                let disc = tagSearch(entity, 'chileancraft:disc:')
                if (disc !== undefined && disc !== 'empty') {
                    const item = dimension.spawnItem(new ItemStack(Items.get(disc), 1, 0), new Location(entity.location.x, entity.location.y, entity.location.z))
                    item.setVelocity(new Vector(0.2 - Math.random() * 0.2, 0.2, 0.2 - Math.random() * 0.2))
                    await entity.runCommandAsync(`stopsound @a[r=64] record.${disc.split(':')[1].replace('music_disc_', '')}`)
                }
                entity.triggerEvent('chileancraft:despawn')

            }
        } catch (err) { console.warn(err) }
    }
}
const beforeUseOnBlocks = {
    'chileancraft:araucaria_door': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (!source.isSneaking) openDoor(dimension, block, blockLocation)
    },
    'chileancraft:esclerofilo_door': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (!source.isSneaking) openDoor(dimension, block, blockLocation)
    },
    'minecraft:jukebox': (blockLocation, source, item, blockFace, dimension, block, data) => {
        const entityLocation = new Location(blockLocation.x + 0.5, blockLocation.y + 0.5, blockLocation.z + 0.5)
        let entityBlock
        let disc = ''
        for (const currentEntity of dimension.getEntitiesAtBlockLocation(blockLocation)) {
            if (currentEntity.typeId === 'chileancraft:jukebox_data') {
                entityBlock = currentEntity
                break
            }
        }
        if (entityBlock === undefined) {
            entityBlock = dimension.spawnEntity('chileancraft:jukebox_data', entityLocation)
            disc = 'empty'
        } else {
            disc = tagSearch(entityBlock, 'chileancraft:disc:')
            entityBlock.removeTag(`chileancraft:disc:"${disc}"`)
        }
        let itemID = item.typeId.split(':')
        try {
            if (itemID[1].startsWith('music_disc_')) { data.cancel = true }
        } catch (err) { }
        if (disc === 'empty' && data.cancel) {
            source.runCommandAsync('replaceitem entity @s slot.weapon.mainhand 0 air')
            disc = item.typeId
            dimension.runCommandAsync(`playsound record.${itemID[1].replace('music_disc_', '')} @a ${entityLocation.x} ${entityLocation.y} ${entityLocation.z}`)
            dimension.runCommandAsync(`titleraw @a[x=${entityLocation.x},y=${entityLocation.y},z=${entityLocation.z},r=64] actionbar {"rawtext":[{"text":"§d"},{"translate":"record.nowPlaying","with": {"rawtext":[{"translate":"item.record_${itemID[1].replace('music_disc_', '')}.desc"}]}}]}`)
        } else if (disc !== 'empty') {
            dimension.spawnItem(new ItemStack(Items.get(disc), 1, 0), blockLocation.above())
            dimension.runCommandAsync(`stopsound @a record.${disc.split(':')[1].replace('music_disc_', '')}`)
            disc = 'empty'
        }
        entityBlock.addTag(`chileancraft:disc:"${disc}"`)
    },
    'minecraft:composter': (blockLocation, source, item, blockFace, dimension, block, data) => {
        let composterPermutation = block.permutation
        if (composterPermutation.getProperty('composter_fill_level').value < 7) {
            if (!item.typeId.startsWith('minecraft:')) {
                data.cancel = true
                const organicLevel = chileanComposter[item.typeId]
                if (organicLevel !== undefined) {
                    dimension.runCommandAsync(`playsound dig.gravel @a ${blockLocation.x} ${blockLocation.y} ${blockLocation.z}`)
                    dimension.runCommandAsync(`particle chileancraft:fertilize ${blockLocation.x} ${blockLocation.y} ${blockLocation.z}`)
                    source.runCommandAsync(`clear @s ${item.typeId} 0 1`)
                    if (Math.random() <= organicLevel) {
                        composterPermutation.getProperty('composter_fill_level').value++
                        if (composterPermutation.getProperty('composter_fill_level').value === 7) composterPermutation.getProperty('composter_fill_level').value++
                        block.setPermutation(composterPermutation)
                    }
                }
            }
        }
    }
}
const beforeUseOnItems = {
    'chileancraft:araucaria_door.item': (blockLocation, source, item, blockFace, dimension, block, data) => {
        placeDoorTest(blockLocation, blockFace, dimension, source, item.typeId, 'dig.wood', 'chileancraft:araucaria_door', data)
    },
    'chileancraft:araucaria_slab': (blockLocation, source, item, blockFace, dimension, block, data) => {
        let blockPlaced
        switch (blockFace) {
            case Direction.up:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                break;
            case Direction.down:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                break;
            case Direction.north:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                break;
            case Direction.south:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                break;
            case Direction.west:
                blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                break;
            case Direction.east:
                blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                break;
        }
        if (replaceableBlocks.includes(blockPlaced.typeId)) source.addTag(`chileancraft:testPlace:${item.typeId}`)
    },
    'chileancraft:araucaria_trapdoor': (blockLocation, source, item, blockFace, dimension, block, data) => {
        let blockPlaced
        switch (blockFace) {
            case Direction.up:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                break;
            case Direction.down:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                break;
            case Direction.north:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                break;
            case Direction.south:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                break;
            case Direction.west:
                blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                break;
            case Direction.east:
                blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                break;
        }
        if (replaceableBlocks.includes(blockPlaced.typeId)) source.addTag(`chileancraft:testPlace:${item.typeId}`)
    },
    'chileancraft:esclerofilo_slab': (blockLocation, source, item, blockFace, dimension, block, data) => {
        let blockPlaced
        switch (blockFace) {
            case Direction.up:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                break;
            case Direction.down:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                break;
            case Direction.north:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                break;
            case Direction.south:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                break;
            case Direction.west:
                blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                break;
            case Direction.east:
                blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                break;
        }
        if (replaceableBlocks.includes(blockPlaced.typeId)) source.addTag(`chileancraft:testPlace:${item.typeId}`)
    },
    'chileancraft:esclerofilo_trapdoor': (blockLocation, source, item, blockFace, dimension, block, data) => {
        let blockPlaced
        switch (blockFace) {
            case Direction.up:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                break;
            case Direction.down:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                break;
            case Direction.north:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                break;
            case Direction.south:
                blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                break;
            case Direction.west:
                blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                break;
            case Direction.east:
                blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                break;
        }
        if (replaceableBlocks.includes(blockPlaced.typeId)) source.addTag(`chileancraft:testPlace:${item.typeId}`)
    },
    'minecraft:painting': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (Math.random() > 0.6 && !(blockFace === Direction.up || blockFace === Direction.down)) {
            data.cancel = true
            if (Array.from(dimension.getEntities({
                type: 'minecraft:player',
                excludeGameModes: [
                    1
                ],
                name: source.name
            }))[0] !== undefined) source.runCommandAsync('clear @s painting 0 1')
            let paintingLocation
            let checkLeftBlock = {
                x: 0,
                z: 0
            }
            let paintingDirection = ''
            let paintingLocationXOffset = 0
            let paintingLocationZOffset = 0
            switch (blockFace) {
                case Direction.north:
                    paintingDirection = 'south'
                    paintingLocationZOffset = 1
                    paintingLocationXOffset = 0.5
                    paintingLocation = blockLocation.offset(0, 0, -1)
                    checkLeftBlock.x = -1
                    break;
                case Direction.south:
                    paintingDirection = 'north'
                    paintingLocationXOffset = 0.5
                    paintingLocation = blockLocation.offset(0, 0, 1)
                    checkLeftBlock.x = 1
                    break;
                case Direction.west:
                    paintingDirection = 'east'
                    paintingLocationXOffset = 1
                    paintingLocationZOffset = 0.5
                    paintingLocation = blockLocation.offset(-1, 0, 0)
                    checkLeftBlock.z = 1
                    break;
                case Direction.east:
                    paintingDirection = 'west'
                    paintingLocationZOffset = 0.5
                    paintingLocation = blockLocation.offset(1, 0, 0)
                    checkLeftBlock.z = -1
                    break;
            }
            let entityPaintingLocation = new Location(paintingLocation.x + paintingLocationXOffset, paintingLocation.y, paintingLocation.z + paintingLocationZOffset)
            let currentCheck = paintingLocation.offset(checkLeftBlock.x, 0, checkLeftBlock.z)
            let currentCheckBehind = blockLocation.offset(checkLeftBlock.x, 0, checkLeftBlock.z)
            let paintingDimensionX = 1
            let paintingDimensionY = 1
            let loopX = true
            let loopY = true
            let limitX = 3
            while (loopY) {
                loopX = true
                while (loopX && paintingDimensionX < limitX) {
                    const getBehindBlock = dimension.getBlock(currentCheckBehind)
                    const getBlock = dimension.getBlock(currentCheck)
                    let isPainting = false
                    for (const currentEntity of Array.from(dimension.getEntitiesAtBlockLocation(currentCheck))) {
                        if (currentEntity.hasTag('painting')) {
                            isPainting = true
                            break;
                        }
                    }
                    if (!(getBehindBlock.typeId === 'minecraft:air' || getBehindBlock.hasTag('plant') || getBlock.typeId !== 'minecraft:air' || isPainting)) {
                        paintingDimensionX++
                        currentCheck = currentCheck.offset(checkLeftBlock.x, 0, checkLeftBlock.z)
                        currentCheckBehind = currentCheckBehind.offset(checkLeftBlock.x, 0, checkLeftBlock.z)
                    } else {
                        loopX = false
                        limitX = paintingDimensionX
                    }
                }
                if (paintingDimensionY < 3) {
                    currentCheck = paintingLocation.offset(0, paintingDimensionY, 0)
                    currentCheckBehind = blockLocation.offset(0, paintingDimensionY, 0,)
                    const getBehindBlock = dimension.getBlock(currentCheckBehind)
                    const getBlock = dimension.getBlock(currentCheck)
                    let isPainting = false
                    for (const currentEntity of Array.from(dimension.getEntitiesAtBlockLocation(currentCheck))) {
                        if (currentEntity.hasTag('painting')) {
                            isPainting = true
                            break;
                        }
                    }
                    if (!(getBehindBlock.typeId === 'minecraft:air' || getBehindBlock.hasTag('plant') || getBlock.typeId !== 'minecraft:air' || isPainting)) {
                        paintingDimensionY++
                        paintingDimensionX = 1
                        currentCheck = currentCheck.offset(checkLeftBlock.x, 0, checkLeftBlock.z)
                        currentCheckBehind = currentCheckBehind.offset(checkLeftBlock.x, 0, checkLeftBlock.z)
                    } else loopY = false
                } else loopY = false

            }
            let painting = dimension.spawnEntity(`chileancraft:chilean_painting<chileancraft:set_${paintingDimensionX}x${paintingDimensionY}_${paintingDirection}>`, entityPaintingLocation)
            painting.runCommandAsync("playsound block.itemframe.add_item @a ~~~ 0.7")
        }
    }
}
const useOnEventsItem = {
    'chileancraft:araucaria_slab': async (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (source.hasTag(`chileancraft:testPlace:${item.typeId}`)) {
            source.removeTag(`chileancraft:testPlace:${item.typeId}`)
            await timer(1)
            let blockPlaced
            switch (blockFace) {
                case Direction.up:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                    break;
                case Direction.down:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                    break;
                case Direction.north:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                    break;
                case Direction.south:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                    break;
                case Direction.west:
                    blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                    break;
                case Direction.east:
                    blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                    break;
            }
            if (blockPlaced.typeId === item.typeId) {
                const permutation = blockPlaced.permutation
                if (blockFace === Direction.north || blockFace === Direction.south) {
                    permutation.getProperty('chileancraft:slab').value = Math.round(data.faceLocationY)
                } else if (blockFace === Direction.west || blockFace === Direction.east) {
                    permutation.getProperty('chileancraft:slab').value = Math.round(data.faceLocationX)
                } else if (blockFace === Direction.down) permutation.getProperty('chileancraft:slab').value = 1
                blockPlaced.setPermutation(permutation)
            }
        }
    },
    'chileancraft:araucaria_trapdoor': async (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (source.hasTag(`chileancraft:testPlace:${item.typeId}`)) {
            source.removeTag(`chileancraft:testPlace:${item.typeId}`)
            await timer(1)
            let blockPlaced
            switch (blockFace) {
                case Direction.up:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                    break;
                case Direction.down:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                    break;
                case Direction.north:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                    break;
                case Direction.south:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                    break;
                case Direction.west:
                    blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                    break;
                case Direction.east:
                    blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                    break;
            }
            if (blockPlaced.typeId === item.typeId) {
                const permutation = blockPlaced.permutation
                if (blockFace === Direction.north || blockFace === Direction.south) {
                    permutation.getProperty('chileancraft:up').value = data.faceLocationY > 0.5
                } else if (blockFace === Direction.west || blockFace === Direction.east) {
                    permutation.getProperty('chileancraft:up').value = data.faceLocationX > 0.5
                } else if (blockFace === Direction.down) permutation.getProperty('chileancraft:up').value = true
                blockPlaced.setPermutation(permutation)
            }
        }
    },
    'chileancraft:esclerofilo_slab': async (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (source.hasTag(`chileancraft:testPlace:${item.typeId}`)) {
            source.removeTag(`chileancraft:testPlace:${item.typeId}`)
            await timer(1)
            let blockPlaced
            switch (blockFace) {
                case Direction.up:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                    break;
                case Direction.down:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                    break;
                case Direction.north:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                    break;
                case Direction.south:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                    break;
                case Direction.west:
                    blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                    break;
                case Direction.east:
                    blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                    break;
            }
            if (blockPlaced.typeId === item.typeId) {
                const permutation = blockPlaced.permutation
                if (blockFace === Direction.north || blockFace === Direction.south) {
                    permutation.getProperty('chileancraft:slab').value = Math.round(data.faceLocationY)
                } else if (blockFace === Direction.west || blockFace === Direction.east) {
                    permutation.getProperty('chileancraft:slab').value = Math.round(data.faceLocationX)
                } else if (blockFace === Direction.down) permutation.getProperty('chileancraft:slab').value = 1
                blockPlaced.setPermutation(permutation)
            }
        }
    },
    'chileancraft:esclerofilo_trapdoor': async (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (source.hasTag(`chileancraft:testPlace:${item.typeId}`)) {
            source.removeTag(`chileancraft:testPlace:${item.typeId}`)
            await timer(1)
            let blockPlaced
            switch (blockFace) {
                case Direction.up:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 1, 0))
                    break;
                case Direction.down:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, -1, 0))
                    break;
                case Direction.north:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, -1))
                    break;
                case Direction.south:
                    blockPlaced = dimension.getBlock(blockLocation.offset(0, 0, 1))
                    break;
                case Direction.west:
                    blockPlaced = dimension.getBlock(blockLocation.offset(-1, 0, 0))
                    break;
                case Direction.east:
                    blockPlaced = dimension.getBlock(blockLocation.offset(1, 0, 0))
                    break;
            }
            if (blockPlaced.typeId === item.typeId) {
                const permutation = blockPlaced.permutation
                if (blockFace === Direction.north || blockFace === Direction.south) {
                    permutation.getProperty('chileancraft:up').value = data.faceLocationY > 0.5
                } else if (blockFace === Direction.west || blockFace === Direction.east) {
                    permutation.getProperty('chileancraft:up').value = data.faceLocationX > 0.5
                } else if (blockFace === Direction.down) permutation.getProperty('chileancraft:up').value = true
                blockPlaced.setPermutation(permutation)
            }
        }
    },
    'minecraft:bone_meal': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (block.typeId === 'minecraft:grass' && data.blockFace === Direction.up && dimension.getBlock(data.blockLocation.offset(0, 1, 0)).typeId === 'minecraft:air') {
            const blockArea = data.blockLocation.offset(-2, -1, -2).blocksBetween(data.blockLocation.offset(2, 1, 2))
            for (const currentLocation of blockArea) {
                if (Math.random() > 0.95) {
                    const getBlock = dimension.getBlock(currentLocation)
                    if (getBlock.typeId === 'minecraft:grass') {
                        const getUpperBlock = dimension.getBlock(currentLocation.offset(0, 1, 0))
                        if (getUpperBlock.typeId === 'minecraft:air') getUpperBlock.setType(MinecraftBlockTypes.get(chileanFlowers[Math.floor((chileanFlowers.length) * Math.random())]))
                    }
                }
            }
        }
    },
    'chileancraft:copper_shovel': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (block.typeId === 'minecraft:grass' && data.blockFace === Direction.up && dimension.getBlock(data.blockLocation.offset(0, 1, 0)).typeId === 'minecraft:air') {
            block.setType(MinecraftBlockTypes.grassPath)
            dimension.runCommandAsync(`playsound fall.grass @a ${data.blockLocation.x} ${data.blockLocation.y} ${data.blockLocation.z} 1 0.75`)
            data.item.getComponent('durability').damage++
            if (data.item.getComponent('durability').damage >= data.item.getComponent('durability').maxDurability) {
                data.source.runCommandAsync('replaceitem entity @s slot.weapon.mainhand 0 air')
                data.source.runCommandAsync('playsound random.break @a ~~~')
            } else data.source.getComponent('inventory').container.setItem(data.source.selectedSlot, data.item)
        }
    },
    'chileancraft:copper_hoe': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (block.hasTag('grass') && data.blockFace === Direction.up && dimension.getBlock(data.blockLocation.offset(0, 1, 0)).typeId === 'minecraft:air') {
            block.setType(MinecraftBlockTypes.farmland)
            dimension.runCommandAsync(`playsound fall.gravel @a ${data.blockLocation.x} ${data.blockLocation.y} ${data.blockLocation.z} 1 0.75`)
            data.item.getComponent('durability').damage++
            if (data.item.getComponent('durability').damage >= data.item.getComponent('durability').maxDurability) {
                data.source.runCommandAsync('replaceitem entity @s slot.weapon.mainhand 0 air')
                data.source.runCommandAsync('playsound random.break @a ~~~')
            } else data.source.getComponent('inventory').container.setItem(data.source.selectedSlot, data.item)
        }
    },
    'chileancraft:copper_axe': (blockLocation, source, item, blockFace, dimension, block, data) => {
        if (vanillaStrippableLogBlock.indexOf(block.typeId) > -1) {
            const logIndex = vanillaStrippableLogBlock.indexOf(block.typeId)
            const oldPermutation = block.permutation
            let newPermutation
            try {
                if (logIndex === 2) {
                    newPermutation = oldPermutation
                    newPermutation.getProperty('stripped_bit').value = true
                } else {
                    if (logIndex < 2) {
                        newPermutation = MinecraftBlockTypes.get(`minecraft:stripped_${oldPermutation.getProperty(`${logIndex ? 'new' : 'old'}_log_type`).value}_log`).createDefaultBlockPermutation()
                        dimension.runCommandAsync(`playsound fall.wood @a ${data.blockLocation.x} ${data.blockLocation.y} ${data.blockLocation.z} 1 0.75`)
                    } else {
                        newPermutation = MinecraftBlockTypes.get(`minecraft:stripped_${block.typeId.split(':')[1]}`).createDefaultBlockPermutation()
                        if (block.typeId.includes('_log')) dimension.runCommandAsync(`playsound fall.wood @a ${data.blockLocation.x} ${data.blockLocation.y} ${data.blockLocation.z} 1 0.75`)
                        else dimension.runCommandAsync(`playsound fall.stem @a ${data.blockLocation.x} ${data.blockLocation.y} ${data.blockLocation.z} 1 0.75`)
                    }
                    newPermutation.getProperty('pillar_axis').value = oldPermutation.getProperty('pillar_axis').value
                }
                block.setPermutation(newPermutation)
                data.item.getComponent('durability').damage++
                if (data.item.getComponent('durability').damage >= data.item.getComponent('durability').maxDurability) {
                    data.source.runCommandAsync('replaceitem entity @s slot.weapon.mainhand 0 air')
                    data.source.runCommandAsync('playsound random.break @a ~~~')
                } else data.source.getComponent('inventory').container.setItem(data.source.selectedSlot, data.item)
            } catch (err) { data.source.runCommandAsync('tell "Claudal VP" ' + err) }
        }
    }
}
//tile entities tick
const redClayPotLiquid = [
    'minecraft:water_bucket',
    'chileancraft:cooking_oil',
    'minecraft:milk_bucket',
    'chileancraft:goat_milk'
]
const redClayPotLiquidContainer = [
    'minecraft:bucket',
    'minecraft:glass_bottle',
    'minecraft:bucket',
    'minecraft:glass_bottle'
]
const redClayPotLiquidDegradation = [
    [
        new ItemStack(Items.get('chileancraft:water_level7'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level6'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level5'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level4'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level3'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level2'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level1'), 1, 0),
        new ItemStack(Items.get('chileancraft:water_level0'), 1, 0)
    ],
    [
        new ItemStack(Items.get('chileancraft:oil_level7'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level6'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level5'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level4'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level3'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level2'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level1'), 1, 0),
        new ItemStack(Items.get('chileancraft:oil_level0'), 1, 0)
    ],
    [
        new ItemStack(Items.get('chileancraft:milk_level7'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level6'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level5'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level4'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level3'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level2'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level1'), 1, 0),
        new ItemStack(Items.get('chileancraft:milk_level0'), 1, 0)
    ],
    [
        new ItemStack(Items.get('chileancraft:goat_milk_level7'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level6'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level5'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level4'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level3'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level2'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level1'), 1, 0),
        new ItemStack(Items.get('chileancraft:goat_milk_level0'), 1, 0)
    ]
]
const redClayPotItems = [
    [
        'minecraft:beef',
        'minecraft:chicken',
        'minecraft:cod',
        'minecraft:mutton',
        'minecraft:pork',
        'minecraft:potato',
        'minecraft:rabbit',
        'minecraft:salmon',
        'chileancraft:cebolla',
        'chileancraft:choclo',
        'chileancraft:pinones',
        'chileancraft:raw_horse',
        'chileancraft:raw_jaiba',
        'chileancraft:raw_macha',
        'chileancraft:raw_merluza'
    ],
    [
        'minecraft:egg',
        'chileancraft:codorniz_egg',
        'chileancraft:raw_sopaipilla',
        'chileancraft:raw_empanada_queso'
    ],
    [
        'chileancraft:cuajo'
    ],
    [
        'chileancraft:cuajo'
    ]
]
const redClayPotCookItems = [
    [
        'minecraft:cooked_beef',
        'minecraft:cooked_chicken',
        'minecraft:cooked_cod',
        'minecraft:cooked_mutton',
        'minecraft:cooked_pork',
        'minecraft:cooked_potato',
        'minecraft:cooked_rabbit',
        'minecraft:cooked_salmon',
        'chileancraft:cooked_cebolla',
        'chileancraft:cooked_choclo',
        'chileancraft:cooked_pinones',
        'chileancraft:cooked_horse',
        'chileancraft:cooked_jaiba',
        'chileancraft:cooked_macha',
        'chileancraft:cooked_merluza'
    ],
    [
        'chileancraft:fried_egg',
        'chileancraft:fried_egg',
        'chileancraft:sopaipilla',
        'chileancraft:empanada_queso'
    ],
    [
        'chileancraft:cheese'
    ],
    [
        'chileancraft:goat_cheese'
    ]
]
const redClayPotLiquidSound = [
    'bucket.empty_water',
    'drip.lava.pointed_dripstone',
    'bucket.empty_water',
    'bucket.empty_water'
]
const redClayPotTick = (entity) => {
    try {
        const dimension = entity.dimension
        const dataID = entity.id.substring(entity.id.length - 10, entity.id.length)
        const scoreboard = world.scoreboard.getObjective(`rcp_${dataID}`)
        let data = {}
        for (const score of scoreboard.getParticipants()) {
            data[`${score.displayName}`] = score
        }
        const /** @type {import("@minecraft/server").InventoryComponentContainer} */ container = entity.getComponent('inventory').container
        for (const i of [1, 2, 3, 4, 5, 6, 7, 9, 10]) container.setItem(i, fakeEmptySlot)
        let liquidSlot = container.getSlot(8)
        let liquidContainerSlot = container.getSlot(11)
        let itemSlot = container.getSlot(0)
        const belowBlock = dimension.getBlock(new BlockLocation(Math.floor(entity.location.x), Math.floor(entity.location.y) - 1, Math.floor(entity.location.z)))
        if (scoreboard.getScore(data.hot)) {
            if (scoreboard.getScore(data.liquid) > -1) {
                const liquidMount = scoreboard.getScore(data.liquid_mount) - 1
                scoreboard.setScore(data.liquid_mount, liquidMount)
                if (liquidMount > 0) {
                    let liquidMountTotal = scoreboard.getScore(data.liquid_mount_total)
                    itemSlot.setItem(new ItemStack(Items.get(redClayPotItems[scoreboard.getScore(data.liquid)][scoreboard.getScore(data.item)]), scoreboard.getScore(data.item_mount), 0))
                    liquidSlot.setItem(redClayPotLiquidDegradation[scoreboard.getScore(data.liquid)][Math.floor(liquidMount / (liquidMountTotal / 8))])
                } else {
                    itemSlot.setItem(new ItemStack(Items.get(redClayPotCookItems[scoreboard.getScore(data.liquid)][scoreboard.getScore(data.item)]), scoreboard.getScore(data.item_mount), 0))
                    scoreboard.setScore(data.liquid, -1)
                    scoreboard.setScore(data.item, -1)
                    scoreboard.setScore(data.item_mount, 0)
                    scoreboard.setScore(data.liquid_mount_total, 0)
                    liquidSlot.setItem(liquidTypeItem)
                }
            } else if (redClayPotLiquid.includes(liquidContainerSlot.typeId)) {
                const liquidType = redClayPotLiquid.indexOf(liquidContainerSlot.typeId)
                if (redClayPotItems[liquidType].includes(itemSlot.typeId)) {
                    entity.runCommandAsync(`playsound ${redClayPotLiquidSound[liquidType]} @a[r=5] ~~~`)
                    scoreboard.setScore(data.liquid, liquidType)
                    scoreboard.setScore(data.item, redClayPotItems[liquidType].indexOf(itemSlot.typeId))
                    scoreboard.setScore(data.item_mount, itemSlot.amount)
                    scoreboard.setScore(data.liquid_mount, Math.floor((itemSlot.amount + 6) * 400 / 7))
                    scoreboard.setScore(data.liquid_mount_total, Math.floor((itemSlot.amount + 6) * 400 / 7))
                    liquidSlot.setItem(redClayPotLiquidDegradation[liquidType][redClayPotLiquidDegradation[liquidType].length - 1])
                    liquidContainerSlot.setItem(new ItemStack(Items.get(redClayPotLiquidContainer[liquidType]), 1, 0))
                } else liquidSlot.setItem(liquidTypeItem)
            } else liquidSlot.setItem(liquidTypeItem)
            if (!(belowBlock.typeId === 'minecraft:fire' || belowBlock.typeId === 'minecraft:soul_fire' || belowBlock.typeId === 'minecraft:campfire' || belowBlock.typeId === 'minecraft:soul_campfire') || ((belowBlock.typeId === 'minecraft:campfire' || belowBlock.typeId === 'minecraft:soul_campfire') && belowBlock.permutation.getProperty('extinguished').value)) scoreboard.setScore(data.hot, 0)
        } else {
            if (scoreboard.getScore(data.liquid) > -1) {
                let liquidMount = scoreboard.getScore(data.liquid_mount)
                let liquidMountTotal = scoreboard.getScore(data.liquid_mount_total)
                itemSlot.setItem(new ItemStack(Items.get(redClayPotItems[scoreboard.getScore(data.liquid)][scoreboard.getScore(data.item)]), scoreboard.getScore(data.item_mount), 0))
            } else {
                liquidSlot.setItem(liquidTypeItem)
            }
            const belowBlock = dimension.getBlock(new BlockLocation(Math.floor(entity.location.x), Math.floor(entity.location.y) - 1, Math.floor(entity.location.z)))
            if (belowBlock.typeId === 'minecraft:fire' || belowBlock.typeId === 'minecraft:soul_fire') scoreboard.setScore(data.hot, 1)
            else if ((belowBlock.typeId === 'minecraft:campfire' || belowBlock.typeId === 'minecraft:soul_campfire') && !belowBlock.permutation.getProperty('extinguished').value) scoreboard.setScore(data.hot, 1)
        }
    } catch (err) { console.warn(err) }
}
world.events.dataDrivenEntityTriggerEvent.subscribe((data) => {
    const dimension = data.entity.dimension
    if (Object.keys(entityEvents).includes(data.id)) entityEvents[`${data.id}`](data.entity, dimension)
})
world.events.beforeDataDrivenEntityTriggerEvent.subscribe((data) => {
    const location = data.entity.location
    const dimension = data.entity.dimension
    switch (data.id) {
        case 'chileancraft:generate_fonda':
            const rotation = Math.random()
            const x = 9 + (rotation > 0.5 ? 0 : 1)
            const z = 9 + (rotation > 0.5 ? 1 : 0)
            const exactLocation = new Location(Math.floor(location.x), Math.floor(location.y) - 2, Math.floor(location.z))
            const blockLocation = new BlockLocation(exactLocation.x, exactLocation.y, exactLocation.z)
            if (Array.from(dimension.getEntities({
                location: exactLocation,
                volume: new BlockAreaSize(x, 6, z),
                excludeTypes: [
                    'minecraft:player',
                    'minecraft:item',
                    'chileancraft:generate_fonda'
                ]
            })).length === 0) {
                const checkBorderList = blockLocation.blocksBetween(blockLocation.offset(x, 6, 0)).concat(blockLocation.offset(0, 0, z).blocksBetween(blockLocation.offset(x, 6, z)), blockLocation.blocksBetween(blockLocation.offset(0, 6, z)), blockLocation.offset(x, 0, 0).blocksBetween(blockLocation.offset(x, 6, z)))
                for (const currentLocation of checkBorderList) {
                    const currentBlock = dimension.getBlock(currentLocation)
                    if (containerBlocks.indexOf(currentBlock.typeId) > -1) break;
                    else if (checkBorderList.indexOf(currentLocation) === checkBorderList.length - 1) createFonda(dimension, exactLocation, blockLocation, x, z)
                }
            }
            break;
    }
})
world.events.beforeItemUseOn.subscribe((data) => {
    const blockLocation = data.blockLocation
    const source = data.source
    const item = data.item
    const blockFace = data.blockFace
    const dimension = source.dimension
    const block = dimension.getBlock(blockLocation)
    if (Object.keys(beforeUseOnBlocks).includes(block.typeId)) beforeUseOnBlocks[`${block.typeId}`](blockLocation, source, item, blockFace, dimension, block, data)
    else if (Object.keys(beforeUseOnItems).includes(item.typeId)) beforeUseOnItems[`${item.typeId}`](blockLocation, source, item, blockFace, dimension, block, data)
})
world.events.itemUseOn.subscribe((data) => {
    const source = data.source
    const dimension = source.dimension
    const blockLocation = data.blockLocation
    const item = data.item
    const blockFace = data.blockFace
    const block = dimension.getBlock(blockLocation)
    if (Object.keys(useOnEventsItem).includes(item.typeId)) useOnEventsItem[`${item.typeId}`](blockLocation, source, item, blockFace, dimension, block, data)
})
world.events.tick.subscribe((data) => {
    if (worldInitialized) {
        playerUpdate()
        for (const redClayPot of dimensionList[0].getEntities({
            type: 'chileancraft:red_clay_pot'
        })) redClayPotTick(redClayPot)
        replaceSlot()
    } else checkWorldInitialized()
})
const checkWorldInitialized = async () => {
    try {
        await world.getDimension('overworld').runCommandAsync('testfor @a')
        worldInitialized = true
    } catch (err) { }
}
world.events.blockBreak.subscribe(({ block, brokenBlockPermutation, dimension, player }) => {
    const inventory = player.getComponent('inventory').container
    const item = inventory.getItem(player.selectedSlot)
    switch (brokenBlockPermutation.type.id) {
        case 'minecraft:tallgrass':
            if (Math.random() > 0.7) dimension.spawnItem(new ItemStack(Items.get(chileanSeeds[Math.round((chileanSeeds.length - 1) * Math.random())]), 1, 0), block.location)
            break;
    }
    if (!(item === undefined || item.typeId.startsWith('minecraft:'))) for (const currentTool of toolType) {
        if (item.typeId.includes(currentTool)) {
            toolDamageFunction[currentTool.replace('_', '')](player, inventory, player.selectedSlot, item, brokenBlockPermutation, block)
            break;
        }
    }
})
world.events.entitySpawn.subscribe(({ entity }) => {
    switch (entity.typeId) {
        case 'chileancraft:red_clay_pot':
            let id = entity.id
            id = id.substring(id.length - 10, id.length)
            world.scoreboard.addObjective(`rcp_${id}`, 'Red Clay Pot')
            entity.runCommandAsync(`scoreboard players set "hot" rcp_${id} 0`)
            entity.runCommandAsync(`scoreboard players set "liquid" rcp_${id} -1`)
            entity.runCommandAsync(`scoreboard players set "cook_state" rcp_${id} 0`)
            entity.runCommandAsync(`scoreboard players set "item" rcp_${id} -1`)
            entity.runCommandAsync(`scoreboard players set "item_mount" rcp_${id} 0`)
            entity.runCommandAsync(`scoreboard players set "liquid_mount" rcp_${id} 0`)
            entity.runCommandAsync(`scoreboard players set "liquid_mount_total" rcp_${id} 0`)
            break;
        case 'chileancraft:mapuche':
            const nameList = ['Ayun', 'Coyam', 'Lautaro', 'Newen', 'Nahuelpan', 'Antilaf', 'Lincoyan', 'Raiquen']
            entity.nameTag = nameList[Math.ceil(nameList.length * Math.random())]
            break;
        case 'chileancraft:chilean_painting':
            entity.addTag('painting')
            break;
        case 'minecraft:painting':
            entity.addTag('painting')
            break;
    }
})